@@ -27,8 +27,6 @@ setClass(Class = "MxExpectationBA81",
 	     dataColumns="numeric",
 	   dims = "character",
 	   verbose = "integer",
-	     output = "list",
-	     debug = "list",
 	     minItemsPerScore = "integer",
 	     weightColumn = "MxCharOrNumber"),
          contains = "MxBaseExpectation")
