@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2015 The OpenMx Project
+#   Copyright 2007-2016 The OpenMx Project
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -51,6 +51,7 @@ setClass(Class = "MxExpectationStateSpace",
 		threshnames = "character",
 		t = "MxCharOrNumber",
 		scores = "logical",
+		AIsZero = "logical",
 		xPredicted = "matrix",
 		yPredicted = "matrix",
 		PPredicted = "matrix",
@@ -82,6 +83,7 @@ setMethod("initialize", "MxExpectationStateSpace",
 		.Object@thresholds <- thresholds
 		.Object@t <- t
 		.Object@scores <- scores
+		.Object@AIsZero <- FALSE
 		.Object@definitionVars <- list()
 		.Object@threshnames <- threshnames
 		return(.Object)
@@ -321,6 +323,14 @@ setMethod("genericExpFunConvert", signature("MxExpectationStateSpace"),
 			checkSSMNotMissing(dMatrix, 'D', omxQuotes(modelname))
 			checkSSMNotMissing(uMatrix, 'u', omxQuotes(modelname))
 		}
+		#
+		# If A is a matrix (not algebra), has all zero values, all NA labels, no free parameters
+		if("MxMatrix" %in% is(aMatrix) && all(aMatrix$values == 0) && all(is.na(aMatrix$labels)) && all(aMatrix$free == FALSE)){
+			.Object@AIsZero <- TRUE
+			# This is used in the backend for continuous time models to allow for constant slope models
+		}
+		#
+		# Do data processing
 		translatedNames <- c(dimnames(cMatrix)[[1]])
 		if (mxDataObject@type == 'raw') {
 			threshName <- .Object@thresholds
@@ -533,15 +543,9 @@ mxKalmanScores <- function(model, data=NA){
 	}
 	x0 <- mxEvalByName(model@expectation@x0, model, compute=TRUE)
 	P0 <- mxEvalByName(model@expectation@P0, model, compute=TRUE)
-	hasDefVars <- FALSE
-	for(i in 1:length(model@matrices)){
-		attempt <- sapply(model@matrices[[i]]$labels, imxIsDefinitionVariable)
-		attempt2 <- grep('[', model@matrices[[i]]$labels, fixed=TRUE)
-		if(any(attempt) || length(attempt2) > 0){
-			hasDefVars <- TRUE
-			break
-		}
-	}
+	
+	hasDefVars <- imxHasDefinitionVariable(model)
+	
 	X.pred <- matrix(0, nrow=nrow(data)+1, ncol=nrow(x0))
 	X.upda <- matrix(0, nrow=nrow(data)+1, ncol=nrow(x0))
 	X.pred[1,] <- x0
@@ -628,14 +632,8 @@ setMethod("genericGenerateData", signature("MxExpectationStateSpace"),
 		R <- mxEvalByName(model@expectation@R, model, compute=TRUE)
 		u <- mxEvalByName(model@expectation@u, model, compute=TRUE)
 		
-		hasDefVars <- FALSE
-		for(i in 1:length(model@matrices)){
-			attempt <- sapply(model@matrices[[i]]$labels, imxIsDefinitionVariable)
-			if(any(attempt)){
-				hasDefVars <- TRUE
-				break
-			}
-		}
+		hasDefVars <- imxHasDefinitionVariable(model)
+		continuousTime <- !single.na(model@expectation@t)
 		
 		x0 <- mxEvalByName(model@expectation@x0, model, compute=TRUE)
 		P0 <- mxEvalByName(model@expectation@P0, model, compute=TRUE)
@@ -647,11 +645,36 @@ setMethod("genericGenerateData", signature("MxExpectationStateSpace"),
 		ty <- matrix(0, ydim, tdim)
 		
 		tx[,1] <- x0
+		oldT <- 0
 		for(i in 2:(tdim+1)){
 			if(hasDefVars){
+				A <- mxEvalByName(model@expectation@A, model, compute=TRUE, defvar.row=i-1)
+				B <- mxEvalByName(model@expectation@B, model, compute=TRUE, defvar.row=i-1)
+				C <- mxEvalByName(model@expectation@C, model, compute=TRUE, defvar.row=i-1)
+				D <- mxEvalByName(model@expectation@D, model, compute=TRUE, defvar.row=i-1)
+				Q <- mxEvalByName(model@expectation@Q, model, compute=TRUE, defvar.row=i-1)
+				R <- mxEvalByName(model@expectation@R, model, compute=TRUE, defvar.row=i-1)
 				u <- mxEvalByName(model@expectation@u, model, compute=TRUE, defvar.row=i-1)
+				newT <- mxEvalByName(model@expectation@t, model, compute=TRUE, defvar.row=i-1)
 			}
-			tx[,i] <- A %*% tx[,i-1] + B %*% u + t(mvtnorm::rmvnorm(1, rep(0, xdim), Q))
+			if(continuousTime){
+				#browser()
+				I <- diag(1, nrow=nrow(A))
+				deltaT <- c(newT - oldT)
+				oldT <- newT
+				# If the A matrix is not all zero, do the full analytic integration
+				if(!identical(all.equal(A, matrix(0, nrow(A), ncol(A))), TRUE)){
+					expA <- OpenMx::expm(A * deltaT)
+					intA <- solve(A) %*% (expA - I)
+				} else {
+					# This is the analytic result when A equals a zero matrix
+					intA <- I*deltaT
+				}
+				xp <- expA %*% tx[,i-1] + intA %*% B %*% u
+			} else {
+				xp <- A %*% tx[,i-1] + B %*% u
+			}
+			tx[,i] <- xp + t(mvtnorm::rmvnorm(1, rep(0, xdim), Q))
 			ty[,i-1] <- C %*% tx[,i-1] + D %*% u + t(mvtnorm::rmvnorm(1, rep(0, ydim), R))
 		}
 		ret <- t(ty)
@@ -660,3 +683,4 @@ setMethod("genericGenerateData", signature("MxExpectationStateSpace"),
 	}
 )
 
+
