@@ -3,7 +3,7 @@ require(OpenMx)
 require(numDeriv)
 
 set.seed(1)
-  
+
 mat1 <- mxMatrix("Full", rnorm(1), free=TRUE, nrow=1, ncol=1, labels="m1", name="mat1")
 
 mu <- 0
@@ -40,16 +40,50 @@ omxCheckCloseEnough(got$output$hessian, 2/sigma)
 omxCheckCloseEnough(got$output$ihessian, sigma/2)
 
 numer <- mxModel(model1, mxComputeSequence(list(
-  mxComputeNumericDeriv(),
+  mxComputeNumericDeriv(checkGradient = FALSE),
   mxComputeReportDeriv())))
 got <- mxRun(numer, silent=TRUE)
 omxCheckCloseEnough(got$output$hessian, 1, 1e-3)
 
 model3 <- mxModel(model1, mxComputeNewtonRaphson())
-got <- mxRun(model3, silent=TRUE)
-omxCheckCloseEnough(got$output$estimate, 0, 1e-4)
-omxCheckCloseEnough(got$output$status$code, 0)
-omxCheckCloseEnough(got$output$iterations, 2L)
+model3 <- mxRun(model3, silent=TRUE)
+omxCheckCloseEnough(model3$output$estimate, 0, 1e-4)
+omxCheckCloseEnough(model3$output$status$code, 0)
+omxCheckCloseEnough(model3$output$iterations, 2L)
+
+model3 <- mxModel(model3, mxComputeSequence(list(
+    mxComputeOnce('fitfunction', 'information', 'hessian'),
+    mxComputeReportDeriv())))
+model3 <- mxRun(model3)
+omxCheckCloseEnough(model3$output$hessian, 1, 1e-3)
+
+#----------------------------------------------------------- code 6
+
+mat1 <- mxMatrix("Full", rnorm(1), free=TRUE, nrow=1, ncol=1, labels="m1", name="mat1")
+obj <- mxAlgebra(abs(mat1) + mat1^2, name = "obj")
+grad <- mxAlgebra(mat1/abs(mat1) + 2 * mat1, name = "grad", dimnames=list("m1", NULL))
+hess <- mxAlgebra(2, name = "hess", dimnames=list("m1", "m1"))
+code6 <- mxModel("code6", mat1, obj, grad, hess,
+                 mxFitFunctionAlgebra("obj", gradient="grad", hessian="hess"))
+m1 <- mxRun(mxModel(code6, mxComputeSequence(list(
+  mxComputeNumericDeriv(checkGradient=FALSE),
+  mxComputeReportDeriv()
+))))
+m2 <- mxRun(mxModel(code6, mxComputeSequence(list(
+  mxComputeOnce('fitfunction', c('fit','gradient','hessian')),
+  mxComputeReportDeriv()
+))))
+omxCheckCloseEnough(c(m1$output$gradient), c(m2$output$gradient), 1e-6)
+omxCheckCloseEnough(c(m1$output$hessian), c(m2$output$hessian), 1e-5)
+
+code6 <- mxRun(mxModel(code6, mxComputeSequence(list(
+  mxComputeNewtonRaphson(),
+  mxComputeReportDeriv()
+))), suppressWarnings = TRUE)
+
+omxCheckEquals(code6$output$status$code, 6)
+omxCheckCloseEnough(code6$output$estimate, 0, 1e-6)
+omxCheckCloseEnough(abs(code6$output$gradient), 1, 1e-6)
 
 #----------------------------------------------------------- multivariate
 
@@ -74,11 +108,6 @@ omxCheckCloseEnough(c(mv1.fit$output$hessian), c(4.86, 1.43, 1.43, 4.71), .01)
 omxCheckCloseEnough(mv1.fit$output$hessian, solve(mv1.fit$output$ihessian), 1e-2)
 
 mv2 <- mxModel(mv1, mxComputeNewtonRaphson())
-mv2.fit <- mxRun(mv2, silent=TRUE)
+mv2.fit <- mxRun(mv2, silent=TRUE, suppressWarnings = TRUE)
+omxCheckEquals(mv2.fit$output$status$code, 6)
 omxCheckCloseEnough(mv2.fit$output$estimate, rep(0, 2), 1)  # probably in a local minimum
-
-mat3 <- mat2
-mat3$free[1,1] <- FALSE
-mv3 <- mxRun(mxModel(mv1, mat3), silent=TRUE)
-omxCheckCloseEnough(c(mv3$output$gradient), mv1.fit$output$gradient["x2"])
-omxCheckCloseEnough(c(mv3$output$hessian), mv1.fit$output$hessian[2,2])
