@@ -44,12 +44,18 @@ biRegModelRaw <- mxModel("FIML Bivariate Regression of y on x1 and x2",
            arrows=2, 
            free=TRUE, values=.2, 
            labels=c("CovX1X2")),
-    mxPath(from="one", to=manifests, 
-           arrows=1, free=TRUE, values=.1, 
-           labels=c("MeanX1", "MeanX2", "MeanY")),
     mxData(observed=multiData1, type="raw")
     )
 
+omxCheckError(mxRun(biRegModelRaw, suppressWarnings=TRUE),
+"The job for model 'FIML Bivariate Regression of y on x1 and x2' exited abnormally with the error message: FIML Bivariate Regression of y on x1 and x2.expectation: raw data observed but no expected means vector was provided. Add something like mxPath(from = 'one', to = manifests) to your model.")
+
+biRegModelRaw <- mxModel(
+  biRegModelRaw,
+  mxPath(from="one", to=manifests, 
+         arrows=1, free=TRUE, values=.1, 
+         labels=c("MeanX1", "MeanX2", "MeanY")))
+
 biRegModelRawOut <- mxRun(biRegModelRaw)
 
 summary(biRegModelRawOut)
