@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2015 The OpenMx Project
+ *  Copyright 2007-2016 The OpenMx Project
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -48,15 +48,15 @@ struct omxExpectation {					// An Expectation
 	/* Fields unique to Expectation Functions */
 	void (*initFun)(omxExpectation *ox);
 	void (*destructFun)(omxExpectation* ox);									// Wrapper for the destructor object
-	void (*computeFun)(omxExpectation* ox, const char *what, const char *how);
+	void (*computeFun)(omxExpectation* ox, FitContext *fc, const char *what, const char *how);
 	void (*printFun)(omxExpectation* ox);										// Prints the appropriate pieces of the expectation
-	void (*populateAttrFun)(omxExpectation* ox, SEXP algebra);					// Add attributes to the result algebra object
+	void (*populateAttrFun)(omxExpectation* ox, SEXP expectation);
 	void (*setVarGroup)(omxExpectation*, FreeVarGroup *);  // TODO remove
 	
 	// componentfun & mutateFun probably take encapsulation a little too seriously.
 	// The Fit function should probably just include the structure definition
 	// for the expectation and access fields directly or through object methods.
-	omxMatrix* (*componentFun)(omxExpectation*, omxFitFunction*, const char*);
+	omxMatrix* (*componentFun)(omxExpectation*, const char*);
 	void (*mutateFun)(omxExpectation*, omxFitFunction*, const char*, omxMatrix*);
 
 	SEXP rObj;																	// Original r Object Pointer
@@ -73,9 +73,6 @@ struct omxExpectation {					// An Expectation
 	omxState* currentState;
 	int expNum;
 
-	omxExpectation *container;
-	std::vector<omxExpectation *> submodels;
-
 	// omxExpectation should not need to know about free variables.
 	FreeVarGroup *freeVarGroup; // TODO remove
 	const char *name;
@@ -96,17 +93,17 @@ omxExpectation* omxExpectationFromIndex(int expIndex, omxState* os);
 	
 
 /* Expectation-specific implementations of matrix functions */
-void omxExpectationRecompute(omxExpectation *ox);
-void omxExpectationCompute(omxExpectation *ox, const char *what, const char *how);
+void omxExpectationRecompute(FitContext *fc, omxExpectation *ox);
+void omxExpectationCompute(FitContext *fc, omxExpectation *ox, const char *what, const char *how);
 
-static inline void omxExpectationCompute(omxExpectation *ox, const char *what)
-{ omxExpectationCompute(ox, what, NULL); }
+static inline void omxExpectationCompute(FitContext *fc, omxExpectation *ox, const char *what)
+{ omxExpectationCompute(fc, ox, what, NULL); }
 
 	omxExpectation* omxDuplicateExpectation(const omxExpectation *src, omxState* newState);
 	
 	void omxExpectationPrint(omxExpectation *source, char* d);					// Pretty-print a (small-form) expectation
 	
-omxMatrix* omxGetExpectationComponent(omxExpectation *ox, omxFitFunction *off, const char* component);
+omxMatrix* omxGetExpectationComponent(omxExpectation *ox, const char* component);
 	
 void omxSetExpectationComponent(omxExpectation *ox, omxFitFunction *off, const char* component, omxMatrix *om);
 
@@ -116,5 +113,6 @@ void omxInitStateSpaceExpectation(omxExpectation *ox);
 void omxInitRAMExpectation(omxExpectation *ox);
 void omxInitExpectationBA81(omxExpectation* oo);
 void omxInitGREMLExpectation(omxExpectation* ox);
+void complainAboutMissingMeans(omxExpectation *off);
 
 #endif /* _OMXEXPECTATION_H_ */
